
function [hat_X,P] = bdu_filter(Y,F,G,H,Q,R,M,E_f,Pi_0,hat_x_0)

% Written by A. H. Sayed, Dec. 1, 1999

% This routine implements the optimal BDU filter
% in time- and measurement form

% The system is assumed to be time-invariant 
% and all observations are passed into the 
% function in a single matrix Y.

% The model is assumed to be of the form
% x_{i+1} = (F+Delta F) x_i + G u_i
%   y_{i} = H x_i + v_i
% with
% Delta F = M S_i E_f

% F is n x n
% G is n x m
% H is p x n
% Q is m x m
% R is p x p
% M is n x q
% E_f is q x n


% Pi_0 is n x n positive-definite

% hat_x_0 is n x 1 initial condition (usually zero)

% Y is p x N is a matrix with  ALL  N observations, each
% of dimension p x 1. That is, each column of Y is an
% observation vector

% hat_X is n x (N+1). Each column contains the resulting
% n x 1 (prediction) state estimate [up to hat{x}_{N}].

%% ALL OTHER FILTERING ROUTINES RETURN (N+1) PREDICTION ESTIMATES
%% THE LAST ESTIMATE (AT TIME N+1) OF THIS ROUTINE IS SET EQUAL, 
%% FOR CONVENIENCE, TO THE FILTERED ESTIMATE.

% P is n x n and it contains the value of the Riccati
% variable at the end of the iterations (approx. steady-state)

% Written by A. H. Sayed, Dec. 6, 1999


[p,N]=size(Y);
[q,n]=size(E_f);
[n,m]=size(G);


P=Pi_0;
hat_X(:,1)=hat_x_0;

P_f=inv(inv(Pi_0)+H'*inv(R)*H);
hat_x_f=P_f*H'*inv(R)*Y(:,1);

A=[H*F H*G]; 

for i=1:N-1
  b=Y(:,i+1)-H*F*hat_x_f;
  Qb=zeros(2*n,2*n); 
  Qb(1:n,1:n)=inv(P_f); 
  Qb(n+1:2*n,n+1:2*n)=inv(Q); 
  W=inv(R); 
  H1=H*M; 
  E_a=[E_f zeros(q,m)]; 
  E_b=-E_f*hat_x_f; 
 
  [z,lambda(i),ct] = bdu_rls(A,b,Qb,W,H1,E_a,E_b); 

  inv_R=inv(R);
  AA=lambda(i)*eye(q,q)-M'*H'*inv_R*H*M;
  AA=inv_R+inv_R*H*M*pinv(AA)*M'*H'*inv_R;
  hat_R=inv(AA);
  
  hat_P_f=inv(inv(P_f)+lambda(i)*E_f'*E_f);
  hat_F=F*(eye(n,n)-lambda(i)*hat_P_f*E_f'*E_f);

  P=F*hat_P_f*F'+G*Q*G';
  R_e=hat_R+H*P*H';
  P_f=P-P*H'*inv(R_e)*H*P;
  hat_X(:,i+1)=hat_F*hat_x_f;
  hat_x_f=hat_X(:,i+1)+P_f*H'*inv(hat_R)*(Y(:,i+1)-H*hat_X(:,i+1)); 
end

hat_X(:,N+1)=hat_x_f;

