
function [hat_X,P] = bdu_filter_sub(Y,F,G,H,Q,R,M,E_f,Pi_0,hat_x_0,alpha)


% This routine  implements the suboptimal BDU filter
% in prediction form.

% The system is assumed to be time-invariant 
% and all observations are passed into the 
% function in a single matrix Y.

% The model is assumed to be of the form
% x_{i+1} = (F+Delta F) x_i + G u_i
%   y_{i} = H x_i + v_i
% with
% Delta F = M S_i E_f

% F is n x n
% G is n x m
% H is p x n
% Q is m x m
% R is p x p
% M is n x q
% E_f is q x n


% Pi_0 is n x n positive-definite
% alpha is a positive scalar usually between (0.1, 2]

% hat_x_0 is n x 1 initial condition (usually zero)

% Y is p x N is a matrix with ALL  N observations, each
% of dimension p x 1. That is, each column of Y is an
% observation vector

% hat_X is n x (N+1). Each column contains the resulting
% n x 1 state estimate [up to hat{x}_{N+1}].

% P is n x n and it contains the value of the Riccati
% variable at the end of the iterations (approx. steady-state)

% Written by A. H. Sayed, Dec. 6, 1999


[p,N]=size(Y);
[q,n]=size(E_f);

P=Pi_0;
hat_X(:,1)=hat_x_0;
lambda=(1+alpha)*norm(M'*H'*inv(R)*H*M);

if norm(H*M) == 0 
  lambda = 0;
  inv_lambda = 0;
  display('Lower bound is zero');
else
 inv_lambda = 1/lambda;
end


AA=H*M*M'*H'*inv_lambda;
hat_R=R-AA;

for i=1:N
 BB1=inv(P)+H'*hat_R*H+lambda*E_f'*E_f;
 BB=eye(n,n)-lambda*inv(BB1)*E_f'*E_f;
 hat_F=F*BB;

 R_e=hat_R + H * P * H';

 hat_X(:,i+1)=hat_F*hat_X(:,i)+hat_F*P*H'*inv(R_e)*(Y(:,i)-H*hat_X(:,i));

 P=F*inv(BB1)*F'+G*Q*G';
end

