
function [x_hat,lambda_hat,min_cost] = bdu_rls(A,b,Q,W,H,E_a,E_b)

% Regularized LS  with uncertainties

% Solves
%  min  max   x'Qx + [(A+dA)x-(b+db)]' W [(A+dA)x-(b+db)]
%  x   dA,db

% subject to [dA db] = H S [E_a E_b]
% for any contraction S.


% A is N x n data matrix
% b is N x 1 vector.

% Q is a positive-definite n x n weighting matrix
% W is a nonnegative-definite N x N weighting matrix

% H    is N x m  (m=n in general)
% E_a  is m x n
% E_b  is m x 1


% x_hat is n x 1 computed solution                       
% lambda_hat is computed scalar parameter (nonnegative)
% min_cost is the resulting optimal cost


% Written by A. H. Sayed, Dec. 1, 1999



 [N,n] = size(A);  % dimensions;
 [N,m] = size(H);

 % Evaluation of lower bound on lambda
 l_bound = norm(H'*W*H);

 % Upper bound is arbitrarily set  (can be changed).
 u_bound = 5*l_bound;     % Extensive simulations have shown 
                          % that the optimal lambda is usually
                          % very close to l_bound or can be taken
                          % as l_bound!


 epsilon=1e-6;
 options=[];            % Default options

 lambda_hat =  fmin('cost',l_bound-epsilon,u_bound,options,A,b,Q,W,H,E_a,E_b);

 min_cost = cost(lambda_hat,A,b,Q,W,H,E_a,E_b);

 Q_hat  = Q + lambda_hat * E_a'*E_a;
 X1_hat = lambda_hat*eye(m,m) - H'*W*H;   
 W_hat  = W + W*H*pinv(X1_hat)*H'*W; 

 X2_hat = Q_hat + A'*W_hat*A;
 x_hat  = inv(X2_hat)*(A'*W_hat*b + lambda_hat*E_a'*E_b);

 

